#include <vector>
#include <iostream>
#include <sstream>
#include <iterator>
#include <algorithm>

// jednoducha sablona funkce
template <class T> 
std::string size (T a) {
    std::stringstream ss;
    ss << "velikost: " << a.size() << "\n";
    return ss.str();
}

int main() {

    // prazdny neinicializovany vektor int nulove velikosti
    std::vector<int> a;
    // vektor int velikosti 10, prvky inicializovany na 0
    std::vector<int> b(10);
    // vektor char velikosti 5
    std::vector<char> c {'a', 'b', 'r', 'e', 'c'};

    // pro zjisteni velikosti vektoru pouzijeme metodu .size() 
    std::cout << "velikost: " << a.size() << std::endl;
    // pro zjednoduseni prace si muzeme vytvorit sablonu
    // pokud se jmenuje size(), je vhodne ji kvalifikovat do globalniho jmenneho prostoru
    // pokud je jednoznacny datovy typ argumentu, neni treba udavat ho jako parametr sablony
    std::cout << ::size(b);
    // plna kvalifikace datoveho typu
    std::cout << ::size<std::vector<char>>(c);
    
    // pomoci metody .reserve() je mozne rezervovat misto pro kontejner
    // zajisti se tak, ze kontejner bude mozne zvetsovat do urcite velikosti
    // bez nutnosti presunu v pameti
    std::cout << "kapacita pred reserve: " << a.capacity() << std::endl;
    a.reserve(100);
    std::cout << a.size() << std::endl;
    std::cout << "kapacita po reserve: " << a.capacity() << std::endl;
    
    // vkladani novych dat na konec vektoru
    for (int i = 0; i < 2; i++) 
        a.push_back(i);

    // pristup pomoci pretizeneho operatoru [] neni kontrolovan na meze
    std::cout << "vypis bez kontroly mezi\n";
    for (int i = 0; i < a.size() + 1; i++) {
        try {
            std::cout << a[i] << " ";
        }
        catch (std::out_of_range & e) {
            std::cout << e.what();
            break;
        }
    }
    std::cout << std::endl;

    // pristup pomoci metody .at() vyvola pri prekroceni mezi vyjimku
    std::cout << "vypis s kontrolou mezi\n"; 
    for (int i = 0; i < a.size() + 1; i++) {
        try {
            std::cout << a.at(i) << " ";
        }
        catch (std::out_of_range & e) {
            std::cout << e.what();
            break;
        }
    }
    std::cout << std::endl;

    // iteratory - ukazatele na prvky kontejneru
    std::vector<char>::iterator I1 = c.begin();
    // zkraceni zapisu pomoci typedef
    typedef std::vector<char>::const_iterator cit;
    cit I2 = c.begin();
    // auto vraci vzdy nekonstatni iterator
    auto I3 = c.begin();

    // prochazeni kontejneru inkrementaci iteratoru
    // iterator je ukazatel, je treba ho dereferencovat
    for (; I3 != c.end(); I3++)
        std::cout << *I3 << " ";
    std::cout << "\n";

    // c++11 - range for
    // pomoci reference lze hodnoty prvky kontejneru menit
    for (auto & ci : b) { // hodnoty z vektoru
        ci = 6;
    }
    // bez reference lze pouze ziskat hodnotu
    for (auto ci : c) { 
        std::cout << ci << " ";
    }
    std::cout << std::endl;

    // vypis na std. vystup pomoci genericke funkce copy a iteratoru vystupniho proudu
    std::copy (c.begin(), c.end(), std::ostream_iterator<char> (std::cout, ", "));
    std::cout << std::endl;
    
    // razeni prvku v kontejneru
    std::sort (c.begin(), c.end());
    std::copy (c.begin(), c.end(), std::ostream_iterator<char> (std::cout, ", "));
    std::cout << std::endl;

    // reverzni razeni prvku v kontejneru
    std::reverse (c.begin(), c.end()); // pripadne sort + rbegin, rend
    std::copy (c.begin(), c.end(), std::ostream_iterator<char> (std::cout, ", "));
    std::cout << std::endl;

    return 0;
}